<?php
// $Id: uts.analysis.inc,v 1.17 2008/08/17 00:59:10 boombatower Exp $
/**
 * @file
 * Provide data analysis pages for the Usability Testing Suite.
 *
 * Copyright 2008 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Menu callback: List the studies that can be analyzed.
 *
 * @return string HTML Output.
 */
function uts_analysis_list() {
  $studies = array_merge(uts_studies_load(UTS_STUDY_STATUS_ACTIVE), uts_studies_load(UTS_STUDY_STATUS_CLOSED));
  $list = array();
  foreach ($studies as $study) {
    if (count(uts_session_load_all($study->nid, TRUE)) > 0) {
      $list[] = t('!title<br />@description',
                   array('!title' => l($study->title, 'admin/uts/analyze/' . $study->nid), '@description' => t($study->body)));
    }
  }

  if (empty($list)) {
    return t('No studies to analyze.');
  }
  return t('<p>Select a study to analyze.</p>!studies', array('!studies' => theme('item_list', $list)));
}

/**
 * Menu callback: Display table of sessions related to study.
 *
 * @param interger $study_nid Current study NID.
 * @return string HTML Output.
 */
function uts_analysis_list_sessions($study_nid) {
  uts_analysis_set_title($study_nid);

  $sessions = uts_session_load_all($study_nid);
  $header = array(t('Name'), t('Status'), t('Total time'), t('Operations'));
  $rows = array();
  foreach ($sessions as $session_id) {
    $row = array();
    $session = uts_session_load($session_id);

    $row[] = $session->session_id;
    $row[] = ($session->complete ? t('complete') : t('incomplete'));
    $row[] = format_interval(uts_data_get_total_time($session->session_id));
    $row[] = uts_render_operations(uts_session_operations($session));
    $rows[] = $row;
  }

  if (empty($rows)) {
    return t('No sessions to analyze.');
  }
  return t('<p>
              Use the analysis tabs to view overal study analysis, or select an individual session from the list
              below for analysis for analysis.
            </p>
            !sessions', array('!sessions' => theme('table', $header, $rows)));
}

/**
 * Menu callback: Display details about session.
 *
 * @param interger $study_nid Current study NID.
 * @param string $session_id Current session ID.
 * @return string HTML Output.
 */
function uts_analysis_session_details($study_nid, $sessiond_id) {
  uts_analysis_set_title($study_nid, $sessiond_id);

  return 'TODO: session details.';
}

/**
 * Menu callback: Returns to sessions overview page.
 *
 * @param interger $study_nid Current study NID.
 * @return string HTML Output.
 */
function uts_analysis_sessions_goto($study_nid) {
  drupal_goto('admin/uts/analyze/' . $study_nid);
}

/**
 * Menu callback: Display particular plugin generated analysis.
 *
 * @param interger $study_nid Current study NID.
 * @param string $plugin Current analysis plug-in.
 * @param string $mode Current analysis mode.
 * @param string $session_id Current session ID.
 * @return string HTML Output.
 */
function uts_analysis($study_nid, $plugin, $mode = NULL, $session_id = NULL) {
  uts_analysis_set_title($study_nid, $session_id);

  if (!uts_analysis_check_parameters($study_nid, $plugin, $mode, $session_id)) {
    return '';
  }

  // Parameters have been checked, display data or perform action.
  // Get timestamp range.
  if ($session_id) {
    list($start, $stop) = uts_data_get_timestamps($session_id); // TODO Provide timestamp filter.
  }
  else {
    list($start, $stop) = array(NULL, NULL);
  }

  // Display data analysis from plug-in.
  $output = uts_analysis_display($plugin, $mode, $study_nid, $session_id, $start, $stop);

  // Add action links.
  $base = "admin/uts/analyze/$study_nid/$plugin/$mode";
  if ($session_id) {
    $base = "admin/uts/analyze/$study_nid/$session_id/$plugin/$mode";
  }
  foreach (array('export') as $action) {
    $output .= l(t('Export'), $base . "/$action");
  }

  return $output;
}

/**
 * Menu callback: Perform an action on analysis item.
 *
 * @param unknown_type $session_id
 * @param interger $study_nid Current study NID.
 * @param string $plugin Current analysis plug-in.
 * @param string $mode Current analysis mode.
 * @param string $action Action to perform.
 * @param string $session_id Current session ID.
 */
function uts_analysis_action($study_nid, $plugin, $mode, $action, $session_id = NULL) {
  if (!uts_analysis_check_parameters($study_nid, $plugin, $mode, $session_id)) {
    return '';
  }
  if (!in_array($action, array('export'))) {
    drupal_set_message(t('Invalid action.'), 'error');
    return '';
  }

  // Perform action.
  if ($action == 'export') {
    uts_analysis_export($plugin, $mode, $study_nid, $session_id, $start, $stop);
    exit();
  }
}

/**
 * Check the parameters to make sure they are valid.
 *
 * @param interger $study_nid Current study NID.
 * @param string $plugin Current analysis plug-in.
 * @param string $mode Current analysis mode.
 * @param string $session_id Current session ID.
 * @return boolean Valid parameters.
 */
function uts_analysis_check_parameters(&$study_nid, &$plugin, &$mode, &$session_id) {
  $studies = array_merge(uts_studies_load(UTS_STUDY_STATUS_ACTIVE), uts_studies_load(UTS_STUDY_STATUS_CLOSED));;
  $sessions = uts_session_load_all($study_nid);
  $analysis_plugins = uts_data_analysis_list();

  $found = FALSE;
  foreach ($studies as $study) {
    if ($study->nid == $study_nid) {
      $found = TRUE;
      break;
    }
  }
  if (!$found) {
    drupal_set_message(t('Invalid study NID.'), 'error');
    return FALSE;
  }

  if ($session_id && !in_array($session_id, $sessions)) {
    drupal_set_message(t('Invalid session ID.'), 'error');
    return FALSE;
  }

  if (!array_key_exists($plugin, $analysis_plugins)) {
    drupal_set_message(t('Invalid plugin.'), 'error');
    return FALSE;
  }

  if ($plugin && !$mode) {
    // Get default mode.
    $plugin_modes = $analysis_plugins[$plugin];
    $mode = $plugin_modes[0];
  }
  else if ($plugin && $mode) {
    $plugin_modes = $analysis_plugins[$plugin];
    if (!in_array($mode, $plugin_modes)) {
      drupal_set_message(t('Invalid plugin mode.'), 'error');
      return FALSE;
    }
  }
  return TRUE;
}

/**
 * Set title to include study and session ID if set.
 *
 * @param interger $study_nid Current study NID.
 * @param string $session_id Current session ID.
 */
function uts_analysis_set_title($study_nid, $session_id = NULL) {
  $study = node_load($study_nid);
  if ($session_id) {
    drupal_set_title(t('Analyze %study: %session', array('%study' => $study->title, '%session' => $session_id)));
  }
  else {
    drupal_set_title(t('Analyze %study', array('%study' => $study->title)));
  }
}

/**
 * Get the display elements from the analysis plug-ins, sort them by weight,
 * and collect the output.
 *
 * @param string $module Data colleciton module from which data will be retreived.
 * @param string $mode The analysis mode.
 * @param interger $study_nid Study NID.
 * @param string $session_id Session ID.
 * @param interger $start_timestamp Start timestamp.
 * @param integer $stop_timestamp Stop timestamp.
 * @return string HTML output.
 */
function uts_analysis_display($plugin, $mode, $study_nid, $session_id, $start_timestamp, $stop_timestamp) {
  $elements = uts_data_analysis_display($plugin, $mode, $study_nid, $session_id, $start_timestamp, $stop_timestamp);
  uasort($elements, 'element_sort');

  // Output all the elements which have been sorted by weight.
  $output = '';
  foreach ($elements as $element) {
    $output .= $element['#data'];
  }
  return $output;
}

/**
 * Collection export files from analysis plug-ins, create ZIP archive, and
 * output to browser.
 *
 * @param string $module Data colleciton module from which data will be retreived.
 * @param string $mode The analysis mode.
 * @param interger $study_nid Study NID.
 * @param string $session_id Session ID.
 * @param interger $start_timestamp Start timestamp.
 * @param integer $stop_timestamp Stop timestamp.
 */
function uts_analysis_export($plugin, $mode, $study_nid, $session_id, $start_timestamp, $stop_timestamp) {
  require_once drupal_get_path('module', 'uts') . '/uts.zip.inc';

  // Get data and generate ZIP file.
  $exports = uts_data_analysis_export($plugin, $mode, $study_nid, $session_id, $start_timestamp, $stop_timestamp);
  $zip = new ZipFile();
  foreach ($exports as $file => $data) {
    $zip->addFile($data, $file);
  }

  // Output file.
  header("Content-Disposition: attachment; filename=\"$plugin-$mode.zip\"");
  header("Content-Type: application/zip");
  header("Content-Transfer-Encoding: binary");

  echo $zip->file();
}

/**
 * Get an array containing the session operation links.
 *
 * @param $session Session object.
 * @return Array of links.
 */
function uts_session_operations($session) {
  $links = array();
  if ($session->complete) {
    $links['analyze'] = array(
      'title' => t('analyze'),
      'href' => "admin/uts/analyze/$session->study_nid/$session->session_id"
    );
  }
  $links['session_destroy'] = array(
    'title' => t('destroy'),
    'href' => "admin/uts/analyze/$session->study_nid/$session->session_id/destroy"
  );
  return $links;
}

/**
 * Menu callback -- ask for confirmation of session destruction then redirect
 * to appropriate overview page.
 */
function uts_analysis_destroy_confirm(&$form_state, $session_id) {
  $redirect = 'admin/uts/analyze/' . arg(3);

  // Inject form values.
  $form['session_id'] = array(
    '#type' => 'value',
    '#value' => $session_id,
  );
  $form['redirect'] = array(
    '#type' => 'value',
    '#value' => $redirect,
  );

  return confirm_form($form,
    t('Are you sure you want to delete %session?', array('%session' => $session_id)),
    isset($_GET['destination']) ? $_GET['destination'] : $redirect,
    t('This action cannot be undone.'),
    t('Delete'),
    t('Cancel')
  );
}

/**
 * Execute session destruction.
 */
function uts_analysis_destroy_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    uts_session_destory($form_state['values']['session_id']);
  }

  // Set the redirect to the pre-determined location.
  $form_state['redirect'] = $form_state['values']['redirect'];
}
