<?php
// $Id: uts.pages.inc,v 1.31 2008/08/17 00:59:10 boombatower Exp $
/**
 * @file
 * Provide interface pages.
 *
 * Copyright 2008 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Page that displays the active studies.
 *
 * @param integer $study_nid Study NID to display.
 * @return string HTML output.
 */
function uts_participate($study_nid = NULL) {
  if ($study_nid) {
    // Display information describing the process before user begins study.
    $study = node_load($study_nid);

    // Make sure study is valid and active.
    if ($study->type != UTS_STUDY) {
      drupal_set_message(t('Not a valid study.'), 'error');
      return '';
    }
    else if ($study->study_status == UTS_STUDY_STATUS_IN_DEVELOPMENT) {
      drupal_set_message(t('Study is not active.'), 'error');
      return '';
    }

    $tasks = uts_tasks_load($study_nid);
    $max_time = 0;
    foreach ($tasks as $task) {
      $max_time += $task->max_time;
    }
    drupal_set_title(t('Participate in @study', array('@study' => $study->title)));

    return t('<p>You have selected %title which is for <i>@role (@level)</i>. The maximum
              time allotted for this study is %time for %count tasks. If you do not finish
              a task in the time allotted that is perfectly fine, or if you would like to
              skip a task you may do so by pressing <i>Done</i>. Once you complete a task
              press the <i>Done</i> button which will present you with the option to
              move on to the next task or come back later.</p>
              <p>You may stop after any idividual task or complete the entire study in one
              sitting, but make sure you have un-interupted time availabe before starting.
              Please do not use reference material for help during the study unless
              instructed to do so.</p>
              <p>To begin read the scenario carefully and press the start button.</p>
              <p><b>Scenario:</b><br />@scenario</p>
              !form',
      array('%title' => $study->title,
            '@role' => uts_get_study_audience_role($study->audience_role),
            '@level' => uts_get_study_audience_level($study->audience_level),
            '%time' => format_interval($max_time),
            '%count' => count($tasks),
            '@scenario' => $study->body,
            '!form' => drupal_get_form('uts_participate_start_form', $study_nid)));
  }

  // Add warning to users who can manage studies since this page may be confusing.
  if (user_access('manage studies')) {
    drupal_set_message(t('This page allows participants to take part in a study. To administer the
                          Usability Testing Suite please goto the <a href="@admin">administration pages</a>.',
                          array('@admin' => url('admin/uts'))), 'warning');
  }

  // Display list of active studies.
  $list = array();
  $studies = array_merge(uts_studies_load(UTS_STUDY_STATUS_ACTIVE), uts_studies_load(UTS_STUDY_STATUS_CLOSED));
  foreach ($studies as $study) {
    // Set status message.
    if ($study->study_status == UTS_STUDY_STATUS_CLOSED) {
      // Make sure open sessions.
      $session_ids = uts_session_load_all($study->nid);
      $incomplete = FALSE;
      foreach ($session_ids as $session_id) {
        $session = uts_session_load($session_id);
        if (!$session->complete) {
          $incomplete = TRUE;
          break;
        }
      }
      if (!$incomplete) {
        // No need to display study if no incomplete sessions.
        continue;
      }
      $status = t(' - resume session only');
    }
    else {
      $status = t(' - open to new participants');
    }

    $list[] = t('<b>!title</b><i>@status</i><br />@body',
      array('!title' => l($study->title, 'uts/participate/' . $study->nid),
            '@status' => $status,
            '@body' => $study->body));
  }

  if (empty($list)) {
    $list[] = t('No active studies.');
  }
  return theme('item_list', $list);
}

/**
 * Participation form.
 */
function uts_participate_start_form(&$form_state, $study_nid) {
  $form = array();
  $form['study_nid'] = array(
    '#type' => 'value',
    '#value' => $study_nid
  );
  $form['participate'] = array(
    '#type' => 'fieldset',
    '#title' => t('Participate'),
    '#description' => t('Take part in the current study.')
  );
  $form['participate']['code'] = array(
    '#type' => 'textfield',
    '#title' => 'Code',
    '#description' => t('Enter the code you were given to resume a previous testing session, otherwise leave this field blank.'),
    '#size' => '20',
    '#default_value' => (isset($_COOKIE['uts_session']) ? $_COOKIE['uts_session'] : '')
  );
  $form['participate']['data_collection'] = array(
    '#type' => 'fieldset',
    '#title' => t('Data collection'),
    '#description' => t('Check data collection requirements and rosolve any issues.'),
    '#tree' => TRUE,
    '#theme' => 'uts_participate_start_form_data_collection'
  );
  $study = node_load($study_nid);
  $requirements = uts_data_client_requirements();
  foreach ($requirements as $module => $requirement) {
    if (array_key_exists($module, $study->data_collection)) {
      $row = array();
      $row['status'] = array(
        '#type' => 'value',
        '#value' => $requirement['status']
      );
      $row['required'] = array(
        '#type' => 'value',
        '#value' => $study->data_collection[$module]['required']
      );
      $row['name'] = array(
        '#type' => 'value',
        '#value' => $requirement['name']
      );
      $row['description'] = array(
        '#type' => 'item',
        '#value' => $requirement['description']
      );

      $form['participate']['data_collection'][$module] = $row;
    }
  }

  $form['participate']['start'] = array(
    '#type' => 'submit',
    '#value' => t('Start')
  );
  return $form;
}

/**
 * Theme data collection requirements into a table.
 *
 * @param array $requirements Fieldset with array of requirements.
 * @return string HTML output.
 */
function theme_uts_participate_start_form_data_collection($requirements) {
  $header = array(t('Status'), t('Name'), t('Description'));
  $rows = array();
  foreach (element_children($requirements) as $module) {
    $requirement = &$requirements[$module];
    $row = array();
    $row[] = theme('image', "misc/watchdog-{$requirement['status']['#value']}.png");
    $row[] = $requirement['name']['#value'];
    $row[] = drupal_render($requirement['description']);

    $rows[] = $row;
  }
  return theme('table', $header, $rows);
}

/**
 * Ensure that code entered is valid.
 */
function uts_participate_start_form_validate($form, &$form_state) {
  if (!empty($form_state['values']['code'])) {
    $session = uts_session_load($form_state['values']['code']);
    if (!$session) {
      form_set_error('code', t('Invalid code.'));
    }
    else if ($session->study_nid != $form_state['values']['study_nid']) {
      form_set_error('code', t('The code is not for this study.'));
      drupal_set_message(t('Please try a different code or go to the related !link.',
                            array('!link' => l('study page', 'uts/participate/' . $study_nid))));
    }
    else if ($session->complete) {
      form_set_error('code', t('This code is for an already completed session.'));
    }
  }

  // Check data collection requirements.
  foreach ($form_state['values']['data_collection'] as $module => $requirement) {
    if ($requirement['status'] == 'error' && $requirement['required']) {
      form_set_error('', t('Data collection plug-in %name is required and has issues that
                            need to be resolved.', array('%name' => $requirement['name'])));
    }
  }
}

/**
 * Create session or resume already open one.
 */
function uts_participate_start_form_submit($form, &$form_state) {
  global $db_prefix;

  if (!empty($form_state['values']['code'])) {
    // Initialize testing session.
    uts_session_restore($form_state['values']['code']);

    // Update data collection plug-ins in testing environment.
    $enable = array();
    $disable = array();
    foreach ($form_state['values']['data_collection'] as $module => $requirement) {
      if ($requirement['status'] != 'error') {
        $enable[] = $module;
      }
      else {
        $disable[] = $module;
      }
    }
    // Enable to proctor to ensure that menu hooks are added appropriately.
    $disable[] = 'uts_proctor'; // TODO If not disabled then menu hooks are not picked up for some reason.
    $enable[] = 'uts_proctor';

    if (!empty($disable)) {
      uts_environment_invoke_callback('disable_modules', $form_state['values']['code'], implode(',', $disable));
    }
    if (!empty($enable)) {
      uts_environment_invoke_callback('enable_modules', $form_state['values']['code'], implode(',', $enable));
    }
  }
  else {
    // Check the number of existing sessions against the maximum participant count.
    $study = node_load($form_state['values']['study_nid']);
    $sessions = uts_session_load_all($study->nid);
    if (count($sessions) >= $study->participant_count) {
      drupal_set_message(t('This study is no longer open to new participants. Please enter a code or try a
                            different study.'), 'error');
      return;
    }

    $session_id = uts_session_create($form_state['values']['study_nid']);

    if ($session_id) {
      drupal_set_message(t('A test environment has been created for you. Please store the following code
                            so that if you choose to stop part way and continue later you may do so.
                            Once you are ready press that <i>Start</i> button again.'));
      drupal_set_message(t('Code: @code', array('@code' => $session_id)));

      // Send e-mail containing session information.
      global $user;
      if (!user_is_anonymous()) {
        $params = array();
        $params['account'] = $user;
        $params['study_nid'] = $study->nid;
        $params['session_id'] = $session_id;
        drupal_mail('uts', 'session_info', $user->mail, user_preferred_language($user), $params);
      }
    }
    else {
      drupal_set_message(t('Failed creating environment, please try again.
                            If this issue continue to persist contact the administrator.'), 'error');
    }

    // Check the number of existing sessions against the maximum participant count. If maximum count reached
    // then close study.
    $sessions = uts_session_load_all($study->nid); // Reload.
    if (count($sessions) >= $study->participant_count) {
      $study->study_status = UTS_STUDY_STATUS_CLOSED;
      $study->revision = FALSE;
      $study = node_submit($study);
      node_save($study);
    }
  }
}

/**
 * Landing page after a participant completes a study.
 *
 * @param boolean $return Remind participant to return in order to finish their study.
 */
function uts_participate_thanks($return = FALSE) {
  if ($return) {
    drupal_set_message(t('Remember to come back and finish your study.'));
  }
  return t('<p>We appreciate you taking the time to participate in our study.</p>
            <p>Have a nice day.</p>');
}

/**
 * Send invitations to people asking them to participate.
 */
function uts_invite(&$form_state) {
  $form = array();

  // Get list of studies for select.
  $studies = uts_studies_load();
  $list = array();
  $list[] = t('--');
  foreach ($studies as $study) {
    $list[$study->nid] = $study->title;
  }

  $form['study_nid'] = array(
    '#type' => 'select',
    '#title' => t('Study'),
    '#options' => $list,
  );
  $form['addresses'] = array(
    '#type' => 'textarea',
    '#title' => t('E-mail addresses'),
    '#description' => t('List of e-mail addresses on separate lines to send the invites to. If the person\'s
                         name is specified using the syntax <em>Name, email@example.com</em> then the parameter
                         !name will use the person\'s name otherwise the e-mail address will be used.'),
    '#rows' => 10
  );
  $form['send'] = array(
    '#type' => 'submit',
    '#value' => t('Send')
  );

  return $form;
}

/**
 * Validate the addresses.
 */
function uts_invite_validate($form, &$form_state) {
  $addresses = explode("\n", $form_state['values']['addresses']);

  $line = 1;
  foreach ($addresses as $address) {
    $parts = explode(',', $address);
    if (count($parts) > 2) {
      form_set_error('addresses', t('To many comma separated parts on line @line.',
                                    array('@line' => $line)));
    }
    $email = trim(count($parts) == 1 ? $parts[0] : $parts[1]);
    if (!valid_email_address($email)) {
      form_set_error('addresses', t('Invalid e-mail address %address on line @line.',
                                    array('%address' => $email, '@line' => $line)));
    }
    $line++;
  }
}

/**
 * Send the invites.
 */
function uts_invite_submit($form, &$form_state) {
  global $user;
  $addresses = explode("\n", $form_state['values']['addresses']);

  // Cycle through addresses.
  foreach ($addresses as $address) {
    $params = array();
    $params['account'] = $user;

    $parts = explode(',', $address);
    if (count($parts) == 1) {
      $params['name'] = $to = trim($parts[0]);
    }
    else {
      $params['name'] = trim($parts[0]);
      $to = t('"@name" <!address>', array('@name' => $params['name'], '!address' => trim($parts[1])));
    }

    if ($form_state['values']['study_nid']) {
      $params['study_nid'] = $form_state['values']['study_nid'];
    }
    drupal_mail('uts', 'invite', $to, user_preferred_language($user), $params);
  }
  drupal_set_message(t('@count invites sent.', array('@count' => count($addresses))));
}

/**
 * Task import page.
 *
 * @param interger $study_nid Study NID to import tasks for.
 * @return array Form output.
 */
function uts_import_tasks(&$form_state, $study_nid = NULL) {
  $form = array();
  $form['study_nid'] = array(
    '#type' => 'value',
    '#value' => $study_nid
  );

  if ($study_nid) {
    uts_set_title($study_nid, 'Import tasks');

    // Only import sources from existing database if they will be related to a study.
    $form['existing_source'] = array(
      '#type' => 'fieldset',
      '#title' => t('Existing Source'),
      '#description' => t('Import tasks from database.'),
      '#weight' => -10
    );
    $form['existing_source']['tasks'] = array(
      '#type' => 'select',
      '#title' => t('Tasks'),
      '#multiple' => TRUE,
      '#weight' => -10
    );
    // Add options.
    $tasks = uts_tasks_load(NULL, TRUE);
    $options = array();
    foreach ($tasks as $task) {
      $options[$task->nid] = $task->title;
    }
    $form['existing_source']['tasks']['#options'] = $options;
    $form['existing_source']['tasks']['#size'] = min(6, count($options));
  }
  else {
    drupal_set_title(t('Import tasks'));
  }

  $form['xml_source'] = array(
    '#type' => 'fieldset',
    '#title' => t('XML Source'),
    '#description' => t('Import tasks from an XML source.'),
    '#weight' => -9
  );
  $form['xml_source']['file'] = array(
    '#type' => 'file',
    '#title' => t('Source'),
    '#description' => t('Correctly formed XML document containing UTS tasks.'),
    '#disabled' => TRUE, // TODO
    '#weight' => -10
  );

  $form['import'] = array(
    '#type' => 'submit',
    '#value' => 'Import',
    '#weight' => -8
  );

  return $form;
}

/**
 * Execute import either from XML or non-related tasks in database.
 */
function uts_import_tasks_submit($form, &$form_state) {
  // Tasks from database.
  if (isset($form_state['values']['tasks']) && count($form_state['values']['tasks']) > 0) {
    uts_tasks_import($form_state['values']['study_nid'], $form_state['values']['tasks']);
  }

  // TODO Tasks from XML.
}

/**
 * Menu callback -- ask for confirmation of task detaching then redirect
 * to appropriate overview page.
 */
function uts_task_detach_confirm(&$form_state, $task) {
  $type = node_get_types('type', $task);
  if ($type->orig_type != UTS_TASK) {
    drupal_set_message(t('Must be task to detach from study.'));
    drupal_goto('admin/uts/tasks');
  }
  $study = node_load($task->study_nid);
  $redirect = 'admin/uts/tasks/' . $study->nid;

  // Inject form values.
  $form['task_nid'] = array(
    '#type' => 'value',
    '#value' => $task->nid,
  );
  $form['redirect'] = array(
    '#type' => 'value',
    '#value' => $redirect,
  );

  return confirm_form($form,
    t('Are you sure you want to detach %task from %study?', array('%task' => $task->title, '%study' => $study->title)),
    isset($_GET['destination']) ? $_GET['destination'] : $redirect,
    t('This action cannot be undone.'),
    t('Detach'),
    t('Cancel')
  );
}

/**
 * Execute task detaching.
 */
function uts_task_detach_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    $task = node_load($form_state['values']['task_nid']);
    $study = node_load($task->study_nid);
    db_query('UPDATE {uts_task}
                SET study_nid = NULL,
                weight = NULL
                WHERE nid = %d', $task->nid);
    drupal_set_message(t('%task was detached from %study.', array('%task' => $task->title, '%study' => $study->title)));
  }

  // Set the redirect to the pre-determined location.
  $form_state['redirect'] = $form_state['values']['redirect'];
}

/**
 * Menu callback -- ask for confirmation of node deletion then redirect
 * to appropriate overview page.
 */
function uts_delete_confirm(&$form_state, $node) {
  $message = t('This action cannot be undone.');

  // Decide on the redirect url.
  $type = node_get_types('type', $node);
  if ($type->orig_type == UTS_STUDY) {
    $redirect = 'admin/uts/studies';
    $message .= t(' Related tasks and session data will also be removed.');
  }
  else if ($type->orig_type == UTS_TASK) {
    // TODO If task is related to study then return to tasks overview for that study.
    $redirect = 'admin/uts/tasks';
  }
  else if ($type->orig_type == UTS_ENVIRONMENT) {
    $redirect = 'admin/uts/environments';
  }

  // Inject form values.
  $form['nid'] = array(
    '#type' => 'value',
    '#value' => $node->nid,
  );
  $form['redirect'] = array(
    '#type' => 'value',
    '#value' => $redirect,
  );

  return confirm_form($form,
    t('Are you sure you want to delete %title?', array('%title' => $node->title)),
    isset($_GET['destination']) ? $_GET['destination'] : $redirect,
    $message,
    t('Delete'),
    t('Cancel')
  );
}

/**
 * Execute node deletion.
 */
function uts_delete_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    node_delete($form_state['values']['nid']);
  }

  // Set the redirect to the pre-determined location.
  $form_state['redirect'] = $form_state['values']['redirect'];
}
